package lab;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

@Service
@ConditionalOnProperty(name = "job.enabled", havingValue = "DownloadPatchJob")
public class DownloadPatchJob implements CommandLineRunner {

    public long startTime = System.currentTimeMillis();

    public int totalSize;

    public Set<Long> successPatchIds;

    public String patchIdsFileName = "patchIds";

    public String patchDirName = "raw\\";

    @Autowired
    public ExecutorService executor;

    public ReadWriteLock dirLock = new ReentrantReadWriteLock();

    public HttpClient httpClient = HttpClients.custom()
            .setMaxConnTotal(100)
            .setMaxConnPerRoute(100)
            .build();

    @Override
    public void run(String... args) throws Exception {
        init();
        while(true) {
            printMessage();
            try {
                if(executor.awaitTermination(15, TimeUnit.SECONDS)){
                    break;
                }
            } catch (InterruptedException e) {

            }
        }
    }

    public void init() throws IOException {
        BufferedReader reader = new BufferedReader(new FileReader(new File(patchIdsFileName)));
        String line;
        Set<Long> patchIdSet = new HashSet<>();
        while ((line = reader.readLine()) != null) {
            if(!"".equals(line)) {
                patchIdSet.add(Long.parseLong(line));
            }
        }
        reader.close();

        File patchDir = new File(patchDirName);
        for (String patchId : patchDir.list()) {
            patchIdSet.remove(Long.parseLong(patchId));
        }

        totalSize = patchIdSet.size();
        successPatchIds = new ConcurrentSkipListSet<>();
        patchIdSet.forEach(patchId -> executor.execute(() -> {
            try{
                String webContent = httpGetContent("https://patchwork.kernel.org/patch/"+patchId);
                String mboxContent = httpGetContent("https://patchwork.kernel.org/patch/"+patchId + "/mbox");
                dirLock.readLock().lock();
                writeContent(patchDirName + patchId + "\\web.html", webContent);
                writeContent(patchDirName + patchId + "\\mbox.patch", mboxContent);
                successPatchIds.add(patchId);
                dirLock.readLock().unlock();
            } catch (Throwable e) {
                System.out.println("error: " + e.getMessage() + ", patchId: " + patchId);
            }
        }));
        executor.shutdown();

        Runtime.getRuntime().addShutdownHook(new Thread(() -> {
            dirLock.writeLock().lock();
            executor.shutdownNow();
            printMessage();
        }));
    }

    public void printMessage(){
        System.out.println("Remain size: " + (totalSize - successPatchIds.size()));
        System.out.println("Used time: " + (System.currentTimeMillis() - startTime) / 1000);
    }

    public String httpGetContent(String url) throws IOException {
        HttpResponse response = httpClient.execute(new HttpGet(url));
        int statusCode = response.getStatusLine().getStatusCode();
        if(statusCode < 200 || statusCode >= 300)
            throw new RuntimeException("status Code: " + statusCode + ", reason: " + response.getStatusLine().getReasonPhrase());
        return EntityUtils.toString(response.getEntity());
    }

    public void writeContent(String filePath, String content) throws IOException {
        File file = new File(filePath);
        file.getParentFile().mkdirs();
        FileWriter writer = new FileWriter(file);
        writer.write(content);
        writer.close();
    }
}
