package lab;

import lab.data.MyAuthor;
import lab.data.Patch;
import lab.data.PatchAuthor;
import lab.repository.MyAuthorRepository;
import lab.repository.PatchAuthorRepository;
import lab.repository.PatchRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

@Service
@ConditionalOnProperty(name = "job.enabled", havingValue = "PatchAuthorJob")
public class PatchAuthorJob implements CommandLineRunner {

    @Autowired
    private PatchRepository patchRepository;

    @Autowired
    private PatchAuthorRepository patchAuthorRepository;

    @Autowired
    private MyAuthorRepository myAuthorRepository;

    private HashMap<String, MyAuthor> emailAuthorMap = new HashMap<>();

    private HashMap<String, MyAuthor> nameAuthorMap = new HashMap<>();

    private List<MyAuthor> authorList = new LinkedList<>();

    private long authorIdGenerator = 1;

    @Override
    public void run(String... args) throws Exception {
        List<Patch> patchList = patchRepository.findAllPatchAuthor();
        List<PatchAuthor> patchAuthorList = new LinkedList<>();
        patchList.forEach(patch -> {
            MyAuthor author = getAuthor(patch.getAuthorName(), patch.getAuthorEmail());
            patchAuthorList.add(new PatchAuthor(patch.getPatchworkId(), author.getMyAuthorId()));
        });
        myAuthorRepository.save(authorList);
        patchAuthorRepository.save(patchAuthorList);
    }

    private MyAuthor getAuthor(String name, String email){
        MyAuthor author = emailAuthorMap.get(email);
        if(author == null){
            author = nameAuthorMap.get(name);
            if(author == null){
                author = new MyAuthor(authorIdGenerator++, name, email);
                authorList.add(author);
                nameAuthorMap.put(name, author);
            }
            author.getAuthorEmail().add(email);
            emailAuthorMap.put(email, author);
        }else if(!author.getAuthorName().contains(name)){
            author.getAuthorName().add(name);
            nameAuthorMap.put(name, author);
        }
        return author;
    }
}
