package lab;

import lab.data.Patch;
import lab.data.PatchSet;
import lab.repository.PatchRepository;
import lab.repository.PatchSetRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@ConditionalOnProperty(name = "job.enabled", havingValue = "PatchSetJob")
public class PatchSetJob implements CommandLineRunner {

    @Autowired
    private PatchRepository patchRepository;

    @Autowired
    private PatchSetRepository patchSetRepository;

    @Override
    public void run(String... args) throws Exception {
        Set<Long> allPatchIds = new HashSet<>(patchRepository.findAllPatchworkId());
        while(!allPatchIds.isEmpty()){
            long aPatchId = allPatchIds.iterator().next();
            allPatchIds.remove(aPatchId);
            PatchSet patchSet = patchSetRepository.findByPatchworkIdsContains(aPatchId);
            if(patchSet != null){
                allPatchIds.removeAll(patchSet.getPatchworkIds());
                continue;
            }
            Patch patch = patchRepository.findOne(aPatchId);
            List<Patch> allPatches = null;
            if(!StringUtils.isEmpty(patch.getInReplyTo())){
                allPatches = patchRepository.findByInReplyTo(patch.getInReplyTo());
            }
            if(allPatches != null){
                Set<Long> patchIds = allPatches.stream()
                        .map(Patch::getPatchworkId)
                        .collect(Collectors.toSet());
                allPatchIds.removeAll(patchIds);
                patchSet = new PatchSet()
                        .setPatchworkIds(patchIds);
                patchSetRepository.save(patchSet);
            }
        }
    }
}
