package lab;

import lab.repository.PatchRepository;
import lab.service.PatchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.ExecutorService;

@Service
@ConditionalOnProperty(name = "job.enabled", havingValue = "ProcessPatchJob")
public class ProcessPatchJob implements CommandLineRunner {

    @Autowired
    private PatchService patchService;

    @Autowired
    private PatchRepository patchRepository;

    @Autowired
    private ExecutorService taskExecutor;

    @Value("${processPatch.patchDir}")
    private String patchDir = "";

    private Set<Long> successPatchIds = ConcurrentHashMap.newKeySet();

    private Set<Long> failPatchIds = ConcurrentHashMap.newKeySet();

    private int totalSize;

    private long startTime;

    private long lastPrintTime = 0L;

    private long minPrintInterval = 15000L;

    @Override
    public void run(String... args) throws Exception {
        startTime = System.currentTimeMillis();
        File dir = new File(patchDir);
        System.out.println("begin list file");
        String[] patchFiles = dir.list();
        System.out.println("begin construct allPatchIds");
        Set<Long> allPatchIds = new HashSet<>(patchFiles.length);
        for (String patchId : patchFiles) {
            allPatchIds.add(Long.valueOf(patchId));
        }
        totalSize = allPatchIds.size();
        System.out.println("begin find all successIds");
        List<Long> successIds = patchRepository.findAllPatchworkId();
        System.out.println("begin add all successIds");
        successPatchIds.addAll(successIds);
        Set<Long> successIdSet = new HashSet<>(successIds);
        System.out.println("begin process patch");
        allPatchIds.stream()
                .filter(patchId -> !successIdSet.contains(patchId))
                .forEach(patchId -> {
                    printProcessInfo();
                    taskExecutor.submit(() -> {
                        try{
                            patchService.processPatch(patchId);
                            successPatchIds.add(patchId);
                        } catch (Exception e){
                            System.out.println("error: " + e.getMessage() + ", patchId: " + patchId);
                            failPatchIds.add(patchId);
                        }
                    });
                });
        System.out.println("patch process task all delivered");
        while(true){
            if(printProcessInfo()) break;
            try {
                Thread.sleep(minPrintInterval);
            } catch (InterruptedException e) {
                System.out.println("sleep interrupted");
                break;
            }
        }
        System.out.println("patch process end");
    }

    public boolean printProcessInfo(){
        long currentTime = System.currentTimeMillis();
        if(currentTime - lastPrintTime < minPrintInterval) return false;
        lastPrintTime = currentTime;
        int successSize = successPatchIds.size();
        int failSize = failPatchIds.size();
        System.out.println("Success: " + successSize / (float) totalSize + "(" + successSize + "/" + totalSize + ")");
        System.out.println("Fail: " + failSize / (float) totalSize + "(" + failSize + "/" + totalSize + ")");
        System.out.println("Used Time: " + (currentTime - startTime) / 1000);
        return successSize + failSize >= totalSize;
    }
}
